/** Bind a socket to a port (Server side). This function creates an
    Socket and binds it to a local port.

   @param port an integer which specifies the port

   @param cqueue an integer how many pending connections queue will
   hold in the waiting queue.

   @return The File Descriptor (FD) which allows access to the bound
   port.
*/

#include <netinet/in.h>

int socket_bind(int port, int cqueue) {
  // FD of the new socket to the bound port
  int sock;
  // address information
  struct sockaddr_in ad;

  // create a socket
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    // cannot created socket, return error
    perror("socket");
    return -1;
  }

  // make ensure that the memory is initiated
  memset(&ad, 0, sizeof(ad));

  // address family: AF_INET: IPv4 Internet protocols
  ad.sin_family = AF_INET;
  // convert and copy port in structure
  ad.sin_port = htons(port);
  // bind to all interfaces -- the port will accept connections to all
  // addresses of the local machine
  ad.sin_addr.s_addr = INADDR_ANY;
  // bind socket
  if (bind(sock, (struct sockaddr *) &ad, sizeof(struct sockaddr)) == -1) {
    // cannot bind, return error
    perror("bind");
    return -1;
  }
  // listen for connections on bound port
  if (listen(sock, cqueue) == -1) {
    // cannot listen, return error
    perror("listen");
    return -1;
  }
  // all right, port is listening. Return the FD as
  // reference for use.
  return sock;
}
